<?php

namespace Drupal\synpay\Plugin\Synpay;

use Drupal\synpay\PluginManager\SynpayPluginBase;
use Drupal\synpay\PluginManager\SynpayPluginInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_checkout\Event\CheckoutEvents;
use Drupal\commerce_order\Event\OrderEvent;
use Symfony\Component\HttpFoundation\Response;

/**
 * Provides Robokassa Gateway.
 *
 * @SynpayAnnotation(
 *   id = "robokassa",
 *   title = @Translation("Robokassa"),
 * )
 */
class SynpayRobokassa extends SynpayPluginBase implements SynpayPluginInterface {
  const ROBOURL = "https://auth.robokassa.ru/Merchant/Index.aspx";
  // TRUE - цена с копейками, FALSE - цена в копейках.
  const PRECISION = TRUE;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->plugin_id = $plugin_id;
    $this->order = isset($configuration['order']) ? $configuration['order'] : FALSE;
    $this->total = isset($configuration['total']) ? $configuration['total'] : FALSE;
    // $this->currency = Currency::RUB;
    $this->lang = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $this->client = NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function pay(string $order, int $total) {
    if ($order != $this->order) {
      \Drupal::messenger()->addError(t("Wrong order number"));
      return;
    }
    $host = \Drupal::request()->getSchemeAndHttpHost();
    $return = "$host/synpay/callback/{$this->plugin_id}";
    $params = [
      'failUrl' => "$host/synpay/callback/{$this->plugin_id}?fail",
    ];
    // $order = Uuid::uuid4()->getHex()->toString();
    $order_amount = str_replace(',', '.', $total);
    $crc = md5("$this->user:$order_amount:$order:$this->pass");
    try {
      $redirect = self::ROBOURL;
      $redirect .= "MerchantLogin=$this->user&OutSum=$order_amount&InvoiceID=$order&SignatureValue=$crc&Description=" . $this->desc;
      if ($this->mode == 1) {
        $redirect .= '&IsTest=1';
      }
      $response = new TrustedRedirectResponse($redirect);
      $response->send();
    }
    catch (ActionException $exception) {
      $message = $exception->getMessage();
      \Drupal::messenger()->addError($message);
      \Drupal::messenger()->addWarning(t('Check login/password to your payment gateway!'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function registerOrder($order_id, $order_amount, $params) {
    $order_amount = $order_amount;
    $receipt = json_encode($params['orderBundle']['cartItems']);
    $crc = md5("$this->user:$order_amount:$order_id:$receipt:$this->pass");
    $data = [
      'MerchantLogin' => $this->user,
      'OutSum' => $order_amount,
      'InvoiceID' => $order_id,
      'Receipt' => $receipt,
      'SignatureValue' => $crc,
      'IsTest' => $this->mode,
      'Description' => $this->desc,
    ];
    return [
      'data' => $data,
      'orderId' => $order_id,
      'url' => self::ROBOURL,
    ];
  }

  /**
   * CallBack.
   */
  public function return(Request $request) {
    $remote_id = $request->get('InvId');
    $result = [
      'remote_id' => $remote_id,
    ];
    $sum = $request->get('OutSum');
    $crc = strtoupper($request->get('SignatureValue'));
    $my_crc = strtoupper(md5("$sum:$remote_id:$this->pass"));
    $payment_storage = \Drupal::entityTypeManager()->getStorage('commerce_payment');
    $payment = $payment_storage->loadByRemoteId($remote_id);
    $result['order_id'] = $payment->getOrderId();
    if ($my_crc == $crc) {
      $result['status'] = 'completed';
    }
    else {
      $result['status'] = 'error';
    }
    return $result;
  }

  /**
   * Pay callback.
   */
  public function callback(Request $request) {
    $remote_id = $request->get('InvId');
    // $result = [
    //   'remote_id' => $remote_id,
    // ];
    $sum = $request->get('OutSum');
    $crc = strtoupper($request->get('SignatureValue'));
    $my_crc = strtoupper(md5("$sum:$remote_id:$this->pass2"));
    $payment_storage = \Drupal::entityTypeManager()->getStorage('commerce_payment');
    $payment = $payment_storage->loadByRemoteId($remote_id);
    $order = $payment->getOrder();
    // $result['order_id'] = $payment->getOrderId();
    if ($my_crc == $crc) {
      // $result['status'] = 'completed';
      $status = 'completed';
      $payment->state = $status;
      $payment->setCompletedTime(time());
      $event = new OrderEvent($order);
      $eventDispatcher = \Drupal::service('event_dispatcher');
      $eventDispatcher->dispatch(CheckoutEvents::COMPLETION, $event);
      $order->getState()->applyTransitionById('place');
      $order->save();
    }
    else {
      // $result['status'] = 'authorization_voided';
      $status = 'authorization_voided';
      $payment->state = $status;
    }
    $payment->setRemoteState($status);
    $payment->save();
    return new Response("OK{$order->id()}", 200);
  }

  /**
   * Commerce Return.
   */
  public function onReturn(string $remote_id) {
    $result = $this->client->getOrderStatus($remote_id);
    switch ($result['orderStatus']) {
      case OrderStatus::DEPOSITED:
        $result['status'] = 'completed';
        break;

      default:
      case OrderStatus::DECLINED:
        $result['status'] = 'authorization_voided';
    }
    return $result;
  }

}
