<?php

namespace Drupal\synpay\PluginManager;

use Drupal\Component\Plugin\PluginBase;

/**
 * Provides an Docker plugin manager.
 *
 * @see Drupal\synpay\PluginManager\SynpayAnnotation
 * @see Drupal\synpay\PluginManager\SynpayPluginInterface
 * @see plugin_api
 */
class SynpayPluginBase extends PluginBase implements SynpayPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->name = FALSE;
    $this->config = \Drupal::config('synpay.settings');
    $this->active = $this->config->get("{$plugin_id}_active");
    $this->mode = $this->config->get("{$plugin_id}_test_mode");
    switch ($this->active) {
      case 1:
        switch ($this->mode) {
          case 0:
            if (!isset($configuration['mode']) || $configuration['mode'] != 'test') {
              switch ($plugin_id) {
                case 'robokassa':
                  $this->user = $this->config->get("{$plugin_id}_login");
                  $this->pass = $this->config->get("{$plugin_id}_live_pass");
                  $this->pass2 = $this->config->get("{$plugin_id}_live_pass2");
                  $this->desc = $this->config->get("{$plugin_id}_description");
                  break;

                case 'paykeeper':
                  $this->secure = $this->config->get("{$plugin_id}_secure");
                default:
                  $this->user = $this->config->get("{$plugin_id}_live_login");
                  $this->pass = $this->config->get("{$plugin_id}_live_token");
                  break;
              }
            }
            else {
              \Drupal::messenger()->addWarning(t("Gateway is in test mode, but the configuration is in live!"));
              \Drupal::messenger()->addWarning(t("Please check configuration."));
              $this->active = '2';
            }
            break;

          case 1:
            if (!isset($configuration['mode']) || $configuration['mode'] == 'test') {
              switch ($plugin_id) {
                case 'robokassa':
                  $this->user = $this->config->get("{$plugin_id}_login");
                  $this->pass = $this->config->get("{$plugin_id}_test_pass");
                  $this->pass2 = $this->config->get("{$plugin_id}_test_pass2");
                  $this->desc = $this->config->get("{$plugin_id}_description");
                  break;

                case 'paykeeper':
                  $this->secure = $this->config->get("{$plugin_id}_secure");
                default:
                  $this->user = $this->config->get("{$plugin_id}_test_login");
                  $this->pass = $this->config->get("{$plugin_id}_test_token");
                  break;
              }
            }
            else {
              \Drupal::messenger()->addWarning(t("Gateway is in live mode, but the configuration is in test!"));
              \Drupal::messenger()->addWarning(t("Please check configuration."));
              $this->active = '2';
            }
            break;
        }
        break;

      default:
        \Drupal::messenger()->addWarning(t("Gateway is not active!"));
        \Drupal::messenger()->addWarning(t("Please check configuration."));
    }
  }

  /**
   * Формирование набора параметров для шлюза.
   */
  public function makeSettings($key) {
    $data = [];
    $config = \Drupal::config('synpay.settings');
    foreach ($this->fieldset as $fieldSuffix) {
      $field = "{$key}_{$fieldSuffix}";
      $data[$fieldSuffix] = $this->active ? $config->get($field, '') : '';
    }
    return (object) $data;
  }

}
